@@ -716,7 +716,7 @@ Script::Name *BayonneSession::attachStart(Event *event)
 	const char *id;
 	Name *scr = event->start.scr;
 	ScriptImage *img = event->start.img;
-	const char *did;
+	const char *did = NULL;
 	bool ref = false;
 	unsigned pri = 0;
 	Line *line;
@@ -852,6 +852,8 @@ final:
 	if(!did)
 		setConst("session.dialed", event->start.dialing);
 
+	setConst("session.auth", "none");
+
 	return scr;
 }
 	
