@@ -769,6 +769,11 @@ void BayonneTSession::sysHeader(const char *tsid)
                 snprintf(buf, sizeof(buf), "CTYPE: %s\n", 
                         getExternal("session.type"));
 		libWrite(buf);
+		cp = getSymbol("session.auth");
+		if(!cp)
+			cp = "none";
+		snprintf(buf, sizeof(buf), "CAUTH: %s\n", cp);
+		libWrite(buf);
 		libWrite("\n");
 
 	}
