@@ -35,7 +35,8 @@ static Keydata::Define driver[] = {
 	{"dtmf", "101"},
 	{"inband", "false"},
 	{"silence", "500"},
-	{"duration", "3600"},
+	{"duration", "10m"},
+	{"invite", "60000"},
 	{"pickup", "2000"},
 	{"hangup", "250"},
 	{"accept", "320"},
@@ -43,6 +44,7 @@ static Keydata::Define driver[] = {
 	{"jitter", "2"},
 	{"filler", "false"},
 	{"urlprefix", "sip:"},
+	{"peering", "friend"},	// default registration type
 #ifdef	WIN32		// use localhost default if not in conf for w32
 	{"interface", "127.0.0.1:5070"},
 #endif
@@ -225,6 +227,10 @@ void Driver::startDriver(void)
 			rtp_port += 10;
 	}
 
+	cp = getLast("interface");
+	if(!cp)
+		cp = "all";
+
 	setString(cbuf, sizeof(cbuf) - 8, getLast("interface"));
 	p = strrchr(cbuf, ':');
 	snprintf(p + 1, 8, "%d", sip_port);
@@ -331,17 +337,50 @@ void Driver::stopDriver(void)
 	}
 }
 
+const char *Driver::assignScript(ScriptImage *img, Line *line)
+{
+	Name *scr = img->getCurrent();
+	char buffer[1024];
+	unsigned idx = 0;
+	const char *cp;
+	char *ep;
+
+	for(;;)
+	{
+		cp = line->args[idx++];
+		if(!cp)
+			break;
+
+		if(ispunct(*cp))
+			return "invalid id in sip assign";
+
+		ep = strchr(cp, ':');
+		if(ep && !stricmp(ep, ":5060"))
+			*ep = 0;
+
+		if(!strnicmp(cp, "sip:", 4))
+			cp += 4;
+
+		snprintf(buffer, sizeof(buffer), "sip.%s", cp);
+		slog.debug("sip: assigning %s to %s", cp, scr->name);
+		img->setPointer(buffer, scr);
+	}
+	return "";
+}
+
 const char *Driver::registerScript(ScriptImage *img, Line *line)
 {
 	Name *scr = img->getCurrent();
 	Registry *reg = (Registry *)img->getMemory(sizeof(Registry));
         const char *cp = ScriptChecks::findKeyword(line, "timeout");
+	const char *lcp;
 	char buf[1024];
 	unsigned len, pos;
 	char *p;
-	const char *uri;
+	char *uri;
+	long mult = 60000;
 
-	if(!ScriptChecks::useKeywords(line, "=uri=timeout=server=proxy=userid=secret"))
+	if(!ScriptChecks::useKeywords(line, "=uri=timeout=server=proxy=userid=secret=type=realm"))
 		return "invalid keywords for sip, use uri, timeout, proxy, userid, secret, port";
 
 	reg->active = false;
@@ -351,6 +390,11 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	reg->hostid = ScriptChecks::findKeyword(line, "proxy");
 	reg->userid = ScriptChecks::findKeyword(line, "userid");
 	reg->secret = ScriptChecks::findKeyword(line, "secret");
+	reg->realm = ScriptChecks::findKeyword(line, "realm");
+	reg->type = ScriptChecks::findKeyword(line, "type");
+
+	if(!reg->type)
+		reg->type = getLast("peering");
 
 	if(!reg->hostid || !*reg->hostid)
 		reg->hostid = ScriptChecks::findKeyword(line, "server");
@@ -388,7 +432,24 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(!cp)
 		cp = getLast("duration");
 
-	reg->duration = 1000 * atol(cp);
+	lcp = cp + strlen(cp) - 1;
+	switch(*lcp)
+	{
+	case 'm':
+	case 'M':
+		mult = 60000;
+		break;
+	case 's':
+	case 'S':
+		mult = 1000;
+		break;
+	case 'h':
+	case 'H':
+		mult = 360000;
+		break;
+	}
+
+	reg->duration = mult * atol(cp);
 	reg->line = line;
 	reg->scr = scr;
 
@@ -415,9 +476,9 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	p = strrchr(buf + 5, ':');
 	if(p)
 		*p = 0;
-        len = strlen(buf) + 1;
-        uri = (const char *)img->getMemory(len);
-        strcpy((char *)uri, buf);
+	len = 8 + strlen(reg->userid) + strlen(reg->hostid);
+        uri = (char *)img->getMemory(len);
+	snprintf(uri, len, "sip:%s@%s", reg->userid, reg->hostid);
 
 	snprintf(buf, sizeof(buf), "sip:%s", reg->hostid);
 
@@ -433,20 +494,21 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	if(!registry)
 		registry = true;
 
+	eXosip_lock();
+
+/*
 	if(reg->userid)
 	{
 		setString(buf, sizeof(buf), reg->userid);
 		p = strchr(buf, '@');
 		if(p)
 			*(p++) = 0;
-		
-		if(eXosip_add_authentication_info(buf, buf, reg->secret, NULL, p))
+
+		if(eXosip_add_authentication_info(buf, buf, reg->secret, NULL, reg->realm))
 			slog.error("sip cannot authenticate with %s", reg->userid);			
 	}
-
-	eXosip_lock();
-
-        reg->regid = eXosip_register_init((char *)reg->uri, (char *)reg->proxy, (char *)reg->contact);
+*/
+	reg->regid = eXosip_register_init((char *)reg->uri, (char *)reg->proxy, (char *)reg->contact);
 
 	if(reg->regid < 0)
 	{
@@ -462,13 +524,19 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	eXosip_unlock();
 
 	reg->setTimer(reg->duration / 2 - 1000);
-	snprintf(buf, sizeof(buf), "sip.%d", reg->regid);
+	snprintf(buf, sizeof(buf), "sipreg.%d", reg->regid);
 	img->setPointer(buf, reg);
 	snprintf(buf, sizeof(buf), "uri.%s", reg->contact);
 	img->setPointer(buf, reg);
 	img->addRegistration(line);
 	line->scr.registry = reg;
 
+	snprintf(buf, sizeof(buf), "sipauth.%s", reg->proxy + 4);
+	p = strchr(buf, ':');
+	if(p && !stricmp(buf, ":5060"))
+		*p = 0;
+	img->setPointer(buf, reg);
+
 	return "";
 }
 
@@ -490,6 +558,61 @@ Session *Driver::getCall(int callid)
 	return NULL;
 }
 
+bool Driver::authenticate(Session *s)
+{
+	ScriptImage *img = NULL;
+	Registry *reg;
+	bool rtn = false;
+	char buf[128];
+	const char *cp = s->getSymbol("session.uri_remote");
+	char *p;
+
+	if(cp)
+		cp = strchr(cp, '@');
+	if(cp)
+		++cp;
+
+	if(!cp)
+		return false;
+		
+	snprintf(buf, sizeof(buf), "sipauth.%s", cp);
+	p = strrchr(buf, ':');
+	if(p && !stricmp(p, ":5060"))
+		*p = 0;
+	img = useImage();
+	reg = (Registry *)img->getPointer(buf);
+	if(!reg)
+		goto done;
+
+	if(!reg->userid)
+		goto done;
+
+        setString(buf, sizeof(buf), reg->userid);
+        p = strchr(buf, '@');
+        if(p)
+		*(p++) = 0; 
+
+	eXosip_lock();
+
+	slog.debug("sip: authenticating %s using %s", cp, reg->userid);
+
+        if(eXosip_add_authentication_info(buf, buf, reg->secret, NULL, reg->realm)) 
+	{
+		slog.error("sip: authentification failed; user=%s", reg->userid);
+		goto unlock;
+	}
+
+	eXosip_retry_call(s->cid);
+	rtn = true;
+
+unlock:
+	eXosip_unlock();
+done:
+	if(img)
+		endImage(img);	
+	return rtn;
+}
+
 void Driver::refresh(void)
 {
 	ScriptRegistry *sreg;
@@ -536,7 +659,9 @@ void Driver::run(void)
 	char buf[1024];
 	char cbuf[256];
 	Registry *reg;
+	Name *scr;
 	ScriptImage *img;
+	const char *cp, *lcp;
 	const char *caller = NULL;
 	const char *dialed = NULL;
 	const char *display = NULL;
@@ -548,6 +673,7 @@ void Driver::run(void)
 	char digit[10];
 	char duration[10];
 	osip_body_t *mbody;
+	const char *auth = "anon";
 
 	Bayonne::waitLoaded();
 	time(&last);
@@ -581,7 +707,7 @@ void Driver::run(void)
 			slog.info("sip: new registration");
 			break;
 		case EXOSIP_REGISTRATION_SUCCESS:
-			snprintf(buf, sizeof(buf), "sip.%d", sevent->rid);
+			snprintf(buf, sizeof(buf), "sipreg.%d", sevent->rid);
 			server->enter();
 			img = server->getActive();
 			reg = (Registry *)img->getPointer(buf);	
@@ -595,15 +721,34 @@ void Driver::run(void)
 			server->leave();
 			break;
 		case EXOSIP_REGISTRATION_FAILURE:
-			snprintf(buf, sizeof(buf), "sip.%d", sevent->rid);
+			snprintf(buf, sizeof(buf), "sipreg.%d", sevent->rid);
 			server->enter();
 			img = server->getActive();
 			reg = (Registry *)img->getPointer(buf);	
+			if(reg && sevent->status_code == 401)
+			{
+			        setString(buf, sizeof(buf), reg->userid);
+        			p = strchr(buf, '@');
+        			if(p)
+                			*(p++) = 0;
+
+			        eXosip_lock();
+
+			        slog.debug("sip: re-authenticating %s using %s", reg->proxy, reg->userid);
+
+			        if(eXosip_add_authentication_info(buf, buf, reg->secret, NULL, reg->realm))
+					slog.error("sip: cannot authenticate");
+				else
+					eXosip_register(sevent->rid, reg->duration / 1000);
+
+				eXosip_unlock();
+			}
+			else 
 			if(reg)
 			{
 				reg->setTimer(1);
 				reg->active = false;
-				slog.info("registration for %s failed", reg->contact);
+				slog.info("registration for %s failed; code=%d, reason=%s", reg->contact, sevent->status_code, sevent->reason_phrase);
 			}
 			else
 				slog.warn("unknown sip registration failed; rid=%s", sevent->rid);
@@ -615,11 +760,27 @@ void Driver::run(void)
 		case EXOSIP_REGISTRATION_TERMINATED:
 			slog.info("sip registration terminated");
 			break;			
+		case EXOSIP_CALL_RINGING:
+			session = getCall(sevent->cid);
+			if(!session)
+				break;
+			event.id = CALL_RINGING;
+			session->postEvent(&event);
+			break;
+                case EXOSIP_CALL_ANSWERED:
+                        session = getCall(sevent->cid);
+                        if(!session)
+                                break;
+                        event.id = CALL_ANSWERED;
+                        session->postEvent(&event);
+                        break;  
 		case EXOSIP_CALL_NEW:
 			dialed = sevent->req_uri;
 
 			snprintf(cbuf, sizeof(cbuf), "%s", sevent->remote_uri);
-			caller = cbuf;
+			caller = strchr(cbuf, '<');
+			if(!caller)
+				caller = cbuf;
 
 			if(*caller == '<')
 			{
@@ -633,7 +794,54 @@ void Driver::run(void)
 			snprintf(buf, sizeof(buf), "uri.%s", sevent->req_uri);
 			img = useImage();
 			reg = (Registry *)img->getPointer(buf);
-			if(!reg)
+			if(reg)
+			{
+				lcp = reg->proxy;
+				if(!strnicmp(lcp, "sip:", 4))
+					lcp += 4;
+				cp = strchr(caller, '@');
+				if(!cp)
+					reg = NULL;
+				if(reg && stricmp(lcp, ++cp))
+					reg = NULL;
+				if(reg)
+					auth="user";
+			}
+
+			if(reg)
+				cp = NULL;
+			else
+	                        cp = strchr(caller, '@');
+
+                        if(cp)
+                        {       
+                                snprintf(buf, sizeof(buf), "sipauth.%s", ++cp);
+                                reg = (Registry *)img->getPointer(buf);  
+				if(reg && stricmp(reg->type, "user"))
+					auth = "peer";
+				else
+					reg = NULL;
+                        } 
+
+			scr = NULL;
+			cp = strchr(caller, '@');
+			if(cp && !reg)
+			{
+				snprintf(buf, sizeof(buf), "sip.%s", ++cp);
+				scr = (Name *)img->getPointer(buf);
+				if(scr)
+					auth = "friend"; // anon peer
+			}
+			cp = dialed;
+			if(cp && !strnicmp(cp, "sip:", 4))
+				cp += 4;
+			if(cp && !reg && !scr)
+			{
+				snprintf(buf, sizeof(buf), "sip.%s", ++cp);
+				scr = (Name *)img->getPointer(buf);
+			}
+
+			if(!reg && !scr)
 			{
 				slog.warn("sip: unknown uri %s", sevent->req_uri);
 sip404:
@@ -643,6 +851,8 @@ sip404:
                     		eXosip_unlock();
 				break;
 			}
+			else if(reg)
+				scr = reg->scr;
 
 			s = getIdle();
 
@@ -690,6 +900,7 @@ sip404:
 			if(!display || !*display)
 				display = caller;
 
+			session->setConst("session.auth", auth);
                         session->setConst("session.dialed", dialed);
                         session->setConst("session.caller", caller);
 			session->setConst("session.display", display);
@@ -755,14 +966,14 @@ sip404:
 				session->setConst("session.subject", sevent->subject);
 
 			slog.debug("sip: starting %s on timeslot %d; connecting %s to %s", 
-				reg->scr->name, session->getSlot(),
+				scr->name, session->getSlot(),
 				session->getSymbol("session.rtp_local"),
 				session->getSymbol("session.rtp_remote"));
 
 			memset(&event, 0, sizeof(event));
 			event.id = START_INCOMING;
 			event.start.img = img;
-			event.start.scr = reg->scr;
+			event.start.scr = scr;
 
 			if(!s->postEvent(&event))
 			{
@@ -775,6 +986,54 @@ sip404:
 			eXosip_answer_call(sevent->did, 180, NULL);
 			eXosip_unlock();
 			break;
+		case EXOSIP_CALL_SERVERFAILURE:
+		case EXOSIP_CALL_GLOBALFAILURE:
+			goto failcode;
+		case EXOSIP_CALL_REQUESTFAILURE:
+			session = getCall(sevent->cid);
+			if(sevent->status_code == 407 && session)
+				if(authenticate(session))
+					break;
+failcode:
+			session = getCall(sevent->cid);
+			switch(sevent->status_code)
+			{
+			case 503:
+				event.id = DIAL_TIMEOUT;
+				break;
+			case 486:
+				event.id = DIAL_BUSY;
+				break;
+			case 404:
+			case 484:
+				event.id = DIAL_INVALID;
+				break;
+			default:
+				event.id = DIAL_FAILED;
+			}
+			if(session)
+				session->postEvent(&event);		
+			slog.debug("sip: failed request; code=%d, reason=%s", sevent->status_code, sevent->reason_phrase);
+			break;
+		case EXOSIP_CALL_STARTAUDIO:
+			session = getCall(sevent->cid);
+			if(!session)
+				break;
+			eXosip_get_localip(buf);
+			session->local_address = buf;
+			session->remote_address = sevent->remote_sdp_audio_ip;
+			session->remote_port = sevent->remote_sdp_audio_port;
+			snprintf(buf, sizeof(buf), "%s:%d",
+				inet_ntoa(session->local_address.getAddress()),
+				session->getLocalPort());
+			session->setConst("session.rtp_local", buf);
+			snprintf(buf, sizeof(buf), "%s:%d",
+				inet_ntoa(session->remote_address.getAddress()),
+				session->getRemotePort());
+			session->setConst("session.rtp_remote", buf);
+			event.id = CALL_CONNECTED;
+			session->postEvent(&event);
+			break;
 		case EXOSIP_CALL_ACK:
 			session = getCall(sevent->cid);
 			if(!session)
@@ -796,6 +1055,7 @@ killit:
 				goto exitcall;
 			break;
 		case EXOSIP_CALL_CLOSED:
+		case EXOSIP_CALL_CANCELLED:
 			session = getCall(sevent->cid);
 			if(!session)
 			{
