@@ -36,6 +36,8 @@ public:
 	const char *portid;
 	const char *userid;
 	const char *secret;
+	const char *type;
+	const char *realm;
 	int regid;
 	bool active;
 };
@@ -72,11 +74,13 @@ public:
 	void startDriver(void);
 	void stopDriver(void);
 	const char *registerScript(ScriptImage *img, Line *line);
+	const char *assignScript(ScriptImage *img, Line *line);
 
 	Session *getCall(int callid);
 
 	void run(void);
 	void refresh(void);
+	bool authenticate(Session *s);
 
 	inline uint8 getDTMFPayload(void)
 		{return dtmf_payload;};
@@ -180,6 +184,8 @@ public:
 
 	void clrAudio(void);
 
+	bool enterSeize(Event *event);
+
 	bool enterRunning(Event *event);
 
 	bool enterHangup(Event *event);
