@@ -114,6 +114,115 @@ void Session::makeIdle(void)
 	BayonneSession::makeIdle();
 }
 
+bool Session::enterSeize(Event *event)
+{
+	char buf[128];
+	const char *cp, *p, *to, *from = NULL;
+	char *sp;
+	const char *svr;
+	osip_message_t *invite;
+	unsigned len;
+	Name *scr = ScriptInterp::getName();
+	Registry *reg = NULL;
+	ScriptImage *img;
+
+	switch(event->id)
+	{
+	case ENTER_STATE:
+		cp = getSymbol("session.dialed");
+		if(!cp)
+		{
+			event->id = DIAL_FAILED;
+			return false;
+		}
+		if(!strchr(cp, '@'))
+		{
+			svr = Driver::sip.getLast("outbound");
+			if(!svr)
+				svr = Bayonne::server->getLast("server");
+			snprintf(buf, sizeof(buf), "sip:%s@%s", cp, svr);
+		}
+		else 
+		{
+			if(!strnicmp(cp, "sip:", 4))
+				p = "";
+			else
+				p = "sip:";
+			snprintf(buf, sizeof(buf), "%s%s", p, cp);
+		} 	
+		
+		setConst("session.uri_remote", buf);	
+		cp = strchr(buf, '@');
+		if(cp)
+			setConst("session.server", ++cp);
+		to = getSymbol("session.uri_remote");
+		
+
+		cp = strchr(to, '@');
+		if(cp)
+		{
+			snprintf(buf, sizeof(buf), "sipauth.%s", ++cp);
+			sp = strrchr(buf, ':');
+			if(sp && !stricmp(sp, ":5060"))
+				*sp = 0;
+			img = useImage();
+			reg = (Registry *)img->getPointer(buf);
+			if(reg)
+			{
+				setConst("session.uri_local", reg->uri);
+				from = getSymbol("session.uri_local");
+			}
+			endImage(img);
+		}			
+
+		if(!from)
+		{
+			snprintf(buf, sizeof(buf) - 2, "sip:%s", scr->name);
+			sp = strchr(buf + 4, ':');
+			if(sp)
+			{
+				*(sp++) = '.';
+				cp = strrchr(scr->name, ':');
+				strcpy(sp, cp + 1);
+			} 
+			len = strlen(buf);
+			buf[len++] = '@';
+			buf[len] = 0;
+			setString(buf + len, sizeof(buf) - len, Driver::sip.getLast("interface"));
+			setConst("session.uri_local", buf);
+			from = getSymbol("session.uri_local");
+		}
+		slog.debug("invite %s from %s", to, from);
+		if(eXosip_build_initial_invite(&invite, (char *)to, (char *)from, NULL, "Hello"))
+		{
+			slog.error("sip: invite invalid");
+			event->id = DIAL_FAILED;
+			return false;
+		}
+		eXosip_lock();
+		snprintf(buf, sizeof(buf), "%d", getLocalPort());
+		offhook = true;
+		cid = eXosip_initiate_call(invite, NULL, NULL, buf);
+		eXosip_unlock();
+		startTimer(atol(Driver::sip.getLast("invite")));
+		return true;
+	case CALL_CONNECTED:
+		startRTP();
+		setRunning();
+		return true;
+	case CALL_ANSWERED:
+		startTimer(driver->getPickupTimer());
+		return true;
+	case CALL_RINGING:
+		stopTimer();
+		return true;
+	case TIMER_EXPIRED:
+		offhook = false;
+		return false;
+	}
+	return false;
+}
+
 bool Session::enterHangup(Event *event)
 {
 	if(event->id == TIMER_EXPIRED)
