@@ -545,6 +545,8 @@ bool Runtime::startSelect(const char *name, const char *dial, const char *caller
 			return startDriver(start_driver, name, dial, caller, display);
 		return false;
 	}
+
+	sel = scr->select;
 	
 	while(sel)
 	{
@@ -689,7 +691,6 @@ bool Runtime::command(char *cmd)
 		caller = strtok_r(NULL, " \t\r\n", &tok);
 		display = strtok_r(NULL, " \t\r\n", &tok);
 		return startSelect(cmd, dial, caller, display);
-		return false;
 	}			
 
 	if(!strnicmp(cmd, "lang", 4))
@@ -765,6 +766,18 @@ skipcerr1:
 		return true;
 	}
 	
+	d = BayonneDriver::get(cmd);
+	if(d)
+	{
+		cmd = strtok_r(NULL, " \t\r\n", &tok);
+		if(!cmd)
+			return false;
+		dial = strtok_r(NULL, " \t\r\n", &tok);
+		caller = strtok_r(NULL, " \t\r\n", &tok);
+		display = strtok_r(NULL, " \t\r\n", &tok);
+		return startDriver(d, cmd, dial, caller, display);
+	}
+
 	slog.error("fifo: unknown command %s", cmd);
 	return false;
 }
