@@ -56,8 +56,10 @@ public:
 class BuildThread : public ScriptThread, public Audio
 {
 private:
-	char pathname[128];
+	char pathname[MAX_PATHNAME];
+	char destname[MAX_PATHNAME];
 	const char **paths;
+	const char *dest;
 	Encoded buffer;
 	Linear lbuffer;
 	Info from, to;
@@ -71,7 +73,7 @@ private:
 	void run(void);
 
 public:
-	BuildThread(ScriptInterp *interp, BayonneAudio *au, Audio::Info *inf, const char **list);
+	BuildThread(ScriptInterp *interp, BayonneAudio *au, Audio::Info *inf, const char *d, const char **list);
 	~BuildThread();
 };
 
@@ -81,6 +83,8 @@ private:
 	const char *src, *dest;
 	FILE *in, *out;
 	char buf[1024];
+	char fn1[MAX_PATHNAME];
+	char fn2[MAX_PATHNAME];
 
 	void run(void);
 public:
@@ -209,6 +213,7 @@ public:
 	bool scrKeyinput(void);
 	bool scrCollect(void);
 	bool scrInput(void);
+	bool scrPath(void);
 	bool scrRead(void);
 	bool scrRoute(void);
 	bool scrErase(void);
@@ -217,6 +222,8 @@ public:
 	bool scrTimeslot(void);
 	bool scrPosition(void);
 	bool scrPathname(void);
+	bool scrReadpath(void);
+	bool scrWritepath(void);
 	bool scrVoicelib(void);
 	bool scrMenu(void);
 	bool scrEndmenu(void);
@@ -246,6 +253,7 @@ public:
 class Checks : public ScriptChecks, public Bayonne
 {
 public:
+	const char *chkPaths(Line *line, ScriptImage *img);
 	const char *chkKey(Line *line, ScriptImage *img);
 	const char *chkCDR(Line *line, ScriptImage *img);
 	const char *chkTonegen(Line *line, ScriptImage *img);
